unit WinAccess;

interface

uses
 Windows, Classes, Controls, Forms, shlobj, shellapi;

type

 TKeyState = (kbOn, kbOff);
 TAccessState = (asVisible, asHiden);
 TWinAccess = class(TComponent)
 private
  SysClasses : array [0..3] of PChar;
  FStartButton : TAccessState;
  FTrayBar : TAccessState;
  FTrayNotify : TAccessState;
  FAppSwitch : TAccessState;
{******************************************************************************}
  FCapsLock : TKeyState;
  FNumLock : TKeyState;
  FScrollLock : TKeyState;
  FSystemKey : TKeyState;
  FHoldCapsLock : Boolean;
  FHoldNumLock : Boolean;
  FHoldScrollLock : Boolean;


  Procedure SetStartButton(v : TAccessState);
  Procedure SetTrayBar(v : TAccessState);
  Procedure SetTrayNotify(v : TAccessState);
  Procedure SetAppSwitch(v : TAccessState);
{******************************************************************************}
  procedure SetCapsLock( value : TKeyState );
  procedure SetNumLock( value : TKeyState );
  procedure SetScrollLock( value : TKeyState );
  procedure SetSystemKey( value : TKeyState );
  procedure SetHoldNumLock(h : Boolean);
  procedure SetHoldCapsLock(h : Boolean);
  procedure SetHoldScrollLock(h : Boolean);
 protected
 public
  constructor Create(AOWner : TComponent); override;
 published
  property StartButton : TAccessState read FStartButton write SetStartButton stored true nodefault;
  property TrayBar : TAccessState read FTrayBar write SetTrayBar stored true nodefault;
  property TrayNotify : TAccessState read FTrayNotify write SetTrayNotify stored true nodefault;
  property AppSwitch : TAccessState read FAppSwitch write SetAppSwitch stored true nodefault;
{******************************************************************************}
  property CapsLock: TKeyState read FCapsLock write SetCapsLock;
  property NumLock: TKeyState read FNumLock write SetNumLock;
  property ScrollLock: TKeyState read FScrollLock write SetScrollLock;
  property SystemKey : TkeyState read FSystemKey write SetSystemKey;
  property HoldNumLock: boolean read FHoldNumLock write SetHoldNumLock;
  property HoldScrollLock: boolean read FHoldScrollLock write SetHoldScrollLock;
  property HoldCapsLock : boolean read FHoldCapsLock write SetHoldCapsLock;
 end;

 procedure Register;
implementation
{$R WinAccess.res}
const
 iTRAYBAR = 0;
 iTRAYNOTIFY = 1;
 iSTARTBUTTON = 2;
 iAPPSWITCHBAR = 3;
var
 HN, HC, HS : Boolean;
 NS, CS, SS : Byte;

{******************************************************************************}

function KeyboardProc(
    code : Integer;	// hook code
    wParam : Integer;	// virtual-key code
    lParam : Integer 	// keystroke-message information
) : LRESULT; stdcall;
var
  KeyState : TKeyBoardState;
begin
 if wParam = VK_NUMLOCK then
 begin
  GetKeyboardState(KeyState);
  if HN then
   KeyState[VK_NUMLOCK] := NS;
  if HC then
   KeyState[VK_CAPITAL] := CS;
  if HS then
   KeyState[VK_SCROLL] := SS;
  SetKeyboardState(KeyState);
  Result := 1;
 end
 else
  Result := 0;
end;

{******************************************************************************}

constructor TWinAccess.Create(AOWner : TComponent);
Var
  KeyState : TKeyBoardState;
  OldValue : integer;
begin
  inherited Create(AOWner);
  FHoldCapsLock := false;
  FHoldNumLock := false;
  FHoldScrollLock := false;
  SetWindowsHookEx(
    WH_KEYBOARD,	// type of hook to install
    KeyboardProc,	// address of hook procedure
    HInstance,	// handle of application instance
    0 	// identity of thread to install hook for
   );
  GetKeyboardState(KeyState);
  Case KeyState[VK_CAPITAL] of
    1 : SetCapsLock(kbOn);
    0 : SetCapsLock(kbOff);
  end;
  case KeyState[VK_NUMLOCK] of
    1 : SetNumLock(kbOn);
    0 : SetNumLock(kbOff);
  end;
  case KeyState[VK_SCROLL] of
    1 : SetScrollLock(kbOn);
    0 : SetScrollLock(kbOff);
  end;
  SystemParametersInfo( SPI_SCREENSAVERRUNNING, 0, @OldValue, 0);
  { Set the SystemKey to its original value, who can teach me a good way? :) }
  case OldValue of
    1 : SetSystemKey(kbOff);
    0 : SetSystemKey(kbOn);
  end;
  SystemParametersInfo( SPI_SCREENSAVERRUNNING, OldValue, @OldValue, 0);

{******************************************************************************}
 SysClasses[0] := 'Shell_TrayWnd';
 SysClasses[1] := 'TrayNotifyWnd';
 SysClasses[2] := 'Button';
 SysClasses[3] := 'MSTaskSwWClass';
end;

Procedure TWinAccess.SetStartButton(v : TAccessState);
var
 wnd : THandle;
begin
 FStartButton := v;
 wnd := FindWindow(SysClasses[iTRAYBAR],nil);
 wnd := FindWindowEx(wnd,HWND(0),SysClasses[iSTARTBUTTON],nil);
 case v of
  asVisible :
   SetWindowPos(wnd, HWND_TOP,0,0,0,0,SWP_NOZORDER+SWP_NOMOVE+SWP_NOSIZE+SWP_SHOWWINDOW);
  asHiden:
   SetWindowPos(wnd, HWND_TOP,0,0,0,0,SWP_NOZORDER+SWP_NOMOVE+SWP_NOSIZE+SWP_HIDEWINDOW);
 end;
end;

Procedure TWinAccess.SetTrayBar(v : TAccessState);
var
 wnd : THandle;
begin
 FTrayBar := v;
 wnd := FindWindow(SysClasses[iTRAYBAR],nil);
 case v of
  asVisible :
   SetWindowPos(wnd, HWND_TOP,0,0,0,0,SWP_NOZORDER+SWP_NOMOVE+SWP_NOSIZE+SWP_SHOWWINDOW);
  asHiden:
   SetWindowPos(wnd, HWND_TOP,0,0,0,0,SWP_NOZORDER+SWP_NOMOVE+SWP_NOSIZE+SWP_HIDEWINDOW);
 end;
end;

Procedure TWinAccess.SetTrayNotify(v : TAccessState);
var
 wnd : THandle;
begin
 FTrayNotify := v;
 wnd := FindWindow(SysClasses[iTRAYBAR],nil);
 wnd := FindWindowEx(wnd,HWND(0),SysClasses[iTRAYNOTIFY],nil);
 case v of
  asVisible :
   SetWindowPos(wnd, HWND_TOP,0,0,0,0,SWP_NOZORDER+SWP_NOMOVE+SWP_NOSIZE+SWP_SHOWWINDOW);
  asHiden:
   SetWindowPos(wnd, HWND_TOP,0,0,0,0,SWP_NOZORDER+SWP_NOMOVE+SWP_NOSIZE+SWP_HIDEWINDOW);
 end;
end;

Procedure TWinAccess.SetAppSwitch(v : TAccessState);
var
 wnd : THandle;
begin
 FAppSwitch := v;
 wnd := FindWindow(SysClasses[iTRAYBAR],nil);
 wnd := FindWindowEx(wnd,HWND(0),SysClasses[iAPPSWITCHBAR],nil);
 case v of
  asVisible :
   SetWindowPos(wnd, HWND_TOP,0,0,0,0,SWP_NOZORDER+SWP_NOMOVE+SWP_NOSIZE+SWP_SHOWWINDOW);
  asHiden:
   SetWindowPos(wnd, HWND_TOP,0,0,0,0,SWP_NOZORDER+SWP_NOMOVE+SWP_NOSIZE+SWP_HIDEWINDOW);
 end;
end;

{*****************************************************************************}
procedure TWinAccess.SetCapsLock( value : TKeyState );
Var
  KeyState : TKeyBoardState;
begin
  CS := not Byte(value);
  GetKeyboardState(KeyState);
  case value of
    kbOn : KeyState[VK_CAPITAL] := 1;
    kbOff : KeyState[VK_CAPITAL] := 0;
  end;
  SetKeyboardState(KeyState);
  FCapsLock := Value;
end;

procedure TWinAccess.SetNumLock( value : TKeyState );
Var
  KeyState : TKeyBoardState;
begin
  NS := not Byte(value);
  GetKeyboardState(KeyState);
  case value of
    kbOn : KeyState[VK_NUMLOCK] := 1;
    kbOff : KeyState[VK_NUMLOCK] := 0;
  end;
  SetKeyboardState(KeyState);
  FNumLock := value;
end;

procedure TWinAccess.SetScrollLock( value : TKeyState );
Var
  KeyState : TKeyBoardState;
begin
  SS := not Byte(value);
  GetKeyboardState(KeyState);
  case value of
    kbOn : KeyState[VK_SCROLL] := 1;
    kbOff : KeyState[VK_SCROLL] := 0;
  end;
  SetKeyboardState(KeyState);
  FScrollLock := value;
end;

procedure TWinAccess.SetSystemKey( value : TKeyState );
var
  NoUse : integer;
begin
  case value of
    kbOn : SystemParametersInfo( SPI_SCREENSAVERRUNNING, 0, @NoUse, 0);
    kbOff : SystemParametersInfo( SPI_SCREENSAVERRUNNING, 1, @NoUse, 0);
  end;
  FSystemKey := value;
end;

procedure TWinAccess.SetHoldNumLock(h : Boolean);
begin
 FHoldNumLock := h;
 HN := h;
end;

procedure TWinAccess.SetHoldCapsLock(h : Boolean);
begin
 FHoldCapsLock := h;
 HC := h;
end;

procedure TWinAccess.SetHoldScrollLock(h : Boolean);
begin
 FHoldScrollLock := h;
 HS := h;
end;


Procedure Register;
begin
 RegisterComponents('FreeVCL', [TWinAccess]);
end;
end.
